dojo.provide("com.ibm.pb.dnd.layout.LayoutColumnSource");

dojo.require("com.ibm.pb.dnd.layout.LayoutSource");

dojo.declare("com.ibm.pb.dnd.layout.LayoutColumnSource", [com.ibm.pb.dnd.layout.LayoutSource], {
	ghostClass: "ibmDndGhost",
	onDndCancel: function(){
		if (this.isDragging) {
			this.inherited(arguments);			
			this.removeGhost();	
			ibmCfg.controllers.dnd.ghostHeight = 0;
		}
	},
	onOverEvent: function(){
		// create ghost in empty container
		this.inherited(arguments);
		this.active = true;
		
		if(this.isDragging && dojo.dnd.manager().target){
			var currentContainer = dojo.dnd.manager().target.node;	
			var controls = dojo.query('.'+ibmCfg.controllers.dnd.CONTROL_CLASS, currentContainer);	
			// check to see if layout container is empty
			if(controls.length == 0){			
				this.emptyContainer = currentContainer;
				if(dojo.query("."+this.ghostClass).length == 0 ){
					// creating ghosts in empty layout container
					this.emptyContainer.appendChild(this.createGhostMarkup(ibmCfg.controllers.dnd.ghostHeight));
				}
			}else{
			// clear the empty layout container object
				this.emptyContainer = null;			
			}
		}
	},
	onOutEvent: function(){
		// remove ghost from empty container
		this.inherited(arguments);
		this.active = false;
		
		if(dojo.query("."+this.ghostClass).length > 0){
			this.removeGhost();
		}
	},
	_addItemClass: function(node, type){
		// create ghost
		if(this.current == node && this.before == type){ return; }
		if(this.isDragging){
			this.createGhost(node, type);
		}
	},
	_removeItemClass: function(node, type){
		// remove ghost
		if(this.isDragging){
			this.removeGhost();
		}
	},
	createGhost: function(node, type){        
		// determines whether to create ghost before or after node
		if(type == "Before"){
			node.insertBefore(this.createGhostMarkup(ibmCfg.controllers.dnd.ghostHeight),
							  node.childNodes[0]);
		}else if (type == "After"){
			node.appendChild(this.createGhostMarkup(ibmCfg.controllers.dnd.ghostHeight));
			if (this.hasHeight(node)) {
			// add height to the node to make room for the ghost ONLY if there is a height set on the node
			// otherwise the node will become too long
				var nodeHeight = dojo.style(node, "height");
				if (nodeHeight > 0) {
				// make sure the node height isn't 0 as it always is the first time this function is called
				// add the ghost height to the height of the node (which is the widget skin)
					ibmCfg.controllers.dnd.ghostAddedHeight = true;
					dojo.style(node, "height", nodeHeight + ibmCfg.controllers.dnd.ghostHeight + "px");
				}
			}
		}
	},
	removeGhost: function(){
		//removes ghost from page
		var ghost = dojo.byId(this.ghostClass);
		if(ghost){
			var parent = ghost.parentNode;
			parent.removeChild(ghost);
			var nodeHeight = dojo.style(parent, "height");
			if (ibmCfg.controllers.dnd.ghostAddedHeight) {
				// if the createGhost() function added some height to the dnd node, 
				// it must now be removed since the ghost was removed
				ibmCfg.controllers.dnd.ghostAddedHeight = false;
				dojo.style(parent, "height", nodeHeight - ibmCfg.controllers.dnd.ghostHeight + "px");
	
			}
		}
	},
	createGhostMarkup: function(height){
		// creates ghost markup for a column container
		var elem = document.createElement("div");
		elem.className = this.ghostClass;
		elem.id = this.ghostClass;
		dojo.style(elem, { height: height+"px"});
		return elem;
	},
	hasHeight: function(node){
		//returns true if the node has a height set on it
		var style = dojo.attr(node, "style");
		if (!style) {
			return false;
		} else if (style.toString().toLowerCase().indexOf("height") >=0) {
			return true;
		} else if (style.height) {
			return true;
		} else {
			return false;
		}
	}
});

com.ibm.pb.dnd.layout.LayoutColumnSource.creator = function(item, hint){
	if(hint=="avatar"){
		var node = dojo.doc.createElement("div");
		if(dojo.dnd.manager().source.current){
			// use the portlet markup as avatar
			node.innerHTML = String(item);
			var elem = dojo.byId(dojo.dnd.manager().source.current.id);
			dojo.style(node, { width: elem.offsetWidth+"px", opacity: 0.5});
		}
		return {node: node, data: item, type: [ibmCfg.controllers.dnd.DND_TYPE]};
	} else {  // this works the same way as the _normalizedCreator/onDropExternal functions in dojo.dnd.Source when there is no given creator function	
		var node = dojo.query("[id~="+ibmCfg.controllers.dnd.currentNode.id+"]")[0];
		if(node){
			var source = ibmCfg.controllers.dnd.dndSources[com.ibm.pb.layout.LayoutParser.extractID(node.parentNode)];
			if(source){
				var t = source.getItem(node.id);
				source.delItem(node.id);
				source.selection = {};  // prevents node from being deleted in dojo.dnd.Selector.deleteSelectedNodes called by onDropExternal()
				return {node: node, data: t.data, type: t.type};
			}
		} 
		return {node:dojo.create("div"),data:item,type:ibmCfg.controllers.dnd.DND_TYPE}
	}
};

com.ibm.pb.dnd.layout.LayoutColumnSource.withHandles = true;