/* Copyright IBM Corp. 2013  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.enabler.io.DynamicResolver");

//<INTERFACE>
/**
 * This provides a system plugin point where extensions can be registered
 * to provide immediate resolution of IO requests for data that can be
 * retrieved from the client.  It allows circumvention of remote requests
 * to serve data that is already available in the system.  It also allows
 * an completely extensible URL naming system such that URLs passed to
 * IO requests can take on formats besides typical ones such as http or https.
 * Extension formats and schemes can be handled by custom plugins to allow
 * an arbitrary mapping of resources on the client to URLs that can be serviced
 * through normal IO requests.
 * The service can be retrieved using the following code: <br>
 * <code>var dynamicResolver = com.ibm.mashups.services.ServiceManager.getService(<br/>
 * &nbsp;&nbsp;&nbsp;&nbsp;com.ibm.mashups.enabler.io.DynamicResolver.SERVICE_NAME);</code><br/>
 * @ibm-spi
 * @ibm-module Base
 */
dojo.declare("com.ibm.mashups.enabler.io.DynamicResolver", null, {
	/**
     * The service name to be used to fetch the service from the ServiceManager
     * @type String
     */
	SERVICE_NAME: "dynamicResolver",
	/**
	 * Registers a plugin for immediate request resolution.
	 * @param {String} id the id of the plugin
	 * @param {Function} matcher the function that returns a validation result
	 * to verify if the plugin can resolve the current request.  Function format: 
	 * <code>Function(String url)</code>.<br>
	 * This function must return a "truthy" value to indicate that it can resolve
	 * this request, or a "falsy" value if it cannot.  Whatever this function returns
	 * will be passed into the plugin's resolver function as the second argument.
	 * @param {Function} resolver the function that resolves the current request
	 * and passes the response data back to the caller.  Function format: 
	 * <code>Function(String url, Anything validationResult)</code>.<br>
	 * This function must return the data for the response.
	 * @param {Boolean} first optional argument to specify if this plugin should
	 * take precedence over all existing plugins if set to true.  If it is false
	 * the plugin will take lowest precedence of existing plugins.  Default is false.
	 * @type void
	 */
	register: function(/*String*/ id, /*Function*/matcher, /*Function*/resolver, /*Boolean?*/first) {
	},
	/**
	 * Unregisters a plugin by its registered id.
	 * @param {String} id the id of the plugin to unregister
	 * @type void
	 */
	unregister: function(/*String*/id) {
	}
});
com.ibm.mashups.enabler.io.DynamicResolver.SERVICE_NAME = "dynamicResolver";
//</INTERFACE>
dojo.require("com.ibm.mm.enabler.io.DynamicResolver");
