/* Copyright IBM Corp. 2013  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.enabler.model.url.ModelUrl");

//<INTERFACE>
/**
 * Interface representing a model URL. This class should be used by the specific model
 * API's in order to generate a valid model URL. The low level classes like ModelRestServiceRequest
 * expect a request to be made with an implementation of the interface. It also allows an easy way
 * to handle URIs. e.g. <br/>
 * <code>&lt;scheme&gt;:&lt;primaryNode&gt;@&lt;secondaryNode&gt;</code><bt/>
 * <code>scheme-specific part = &lt;primaryNode&gt;@&lt;secondaryNode&gt;
 * 
 * @ibm-spi
 * @ibm-module Base
*/
dojo.declare("com.ibm.mashups.enabler.model.url.ModelUrl", null, {
        /**
        * @private
        */
        constructor: function () {
        },
        /**
        * Sets the nodes of the scheme-specific part.
        * @param {JSONArray} nodes an array containing the nodes of the scheme-specific part in a JSON notation way.<br/>
        *                          The ordering of the array elements specifies the position of the node, meaning
        *                          the array element 0 will become node 1 and so on and so forth.<br/><br/>
        * valid JSON attributes are: <br/>
        * <ul>
        * <li>value <i>{String} [mandatory]</i></li>
        * <li>isID <i>{boolean} [optional, default: true]</i></li>
        * <li>subModel <i>{String} [optional]</i></li>
        * </ul><br/>
        * example for &quot;nodes&quot;: <br/>
        * <code>
        * [ {<br/> 
        * value: &quot;100&quot;,<br/>
        *   isID: false,<br/>
        *   subModel: &quot;member&quot;<br/>
        * } ]
        * </code>
        * @type {void}
        */ 
        setNodes: function(nodes) {
        },
        /**
        * Sets the scheme-specific part of the unerlying URI.
        * @param {String} value the value for the scheme-specifc part. (Must not be <code>null</code>)
        * @type {void} 
        */
        setSchemeSpecificPart: function (value) {
        },
        /**
        * Returns the scheme-specific part of the underlying URI.
        * @type {String}
        * @return the scheme-specific part. If a scheme-specific part was set through <code>setSchemeSpecificPart</code>
        *         the raw value will be returned. If the SSP was set by using <code>setNodes</code>,
        *         the converted part will be returned. If neither a node nor the raw SSP was set, 
        *         this method returns <code>null</code>.
        */
        getSchemeSpecificPart: function() {
            return null;
        },
        /**
        * Adds the value of the specified parameter.
        * @param {String} name the name of the parameter; must not be <code>null</code>.
        * @param {String} value the value of the parameter. Valid is only a single String. (Must not be <code>null</code>)
        * @type {void} 
        */
        addParameter: function (name, value) {
        },
        /**
        * Sets the value(s) of the specified parameter.
        * @param {String} name the name of the parameter; must not be <code>null</code>.
        * @param {String} value the value of the parameter. Valid are either a single String or a String array. (Must not be <code>null</code>)
        * @type {void} 
        */
        setParameter: function (name, value) {
        },
        /**
        * Returns the value(s) of the specified parameter.
        * @param {String} name the name of the parameter; must not be <code>null</code>.
        * @type {String}
        * @return the value(s) of the specified parameter or <code>null</code> if the specified parameter wasn't found. In case the parameter has one value
        *   the return type is String. For multiple values a String array is returned.
        */
        getParameter: function (name) {
            return null;
        },
        /**
        * Returns the parameters as an object. <b>Note: this must be used read-only</b>
        * @type {JSON}
        * @return an object containing all parameters in the format { &lt;name&gt;: [&lt;value&gt;, &lt;value&gt;, ... ], ... }
        */
        getParameters: function() {
            return null;
        },
        /**
        * Returns the absolute URL of this object.
        * @type {String}
        * @return the absolute URL of this object. e.g. <code>http://myhost/contenthandler?uri=acme:id:myid</code>
        *         or <code>/contenthandler?uri=acme:id:myid</code>
        */
        getAbsoluteURL: function () {
        	return null;
        },
        /**
        * The same as <code>getAbsoluteURL</code> with the difference that this URL is (if needed) converted to a proxified URL
        * @type {String}
        * @return the proxified absolute URL. e.g. <code>http://acme.com/proxy/http/myhost/contenthandler?uri=acme:id:myid</code>
        */
        getProxifiedAbsoluteURL: function () {
            return null;
        },
        /**
        * Returns the model URI of this object. If the complete link is e.g. <code>/mum/contenthandler?uri=acme:id:myid</code>,
        * this method will return <code>acme:id:myid</code>.
        * @type {String}
        * @return the URI portion of this Model URL. 
        */
        getModelURI: function () {
            return null;
        }
    }
);
//</INTERFACE>
